@@ -1,8 +1,11 @@
+import torch
 from torch import nn, Tensor
-from typing import List, Union
+from typing import List, Union, Iterable, Optional
 
 __all__ = ["ApplySoftmaxTo", "ApplySigmoidTo", "Ensembler", "PickModelOutput"]
 
+from pytorch_toolbelt.inference.tta import _deaugment_averaging
+
 
 class ApplySoftmaxTo(nn.Module):
     def __init__(self, model: nn.Module, output_key: Union[str, List[str]] = "logits", dim=1, temperature=1):
@@ -55,40 +58,35 @@ class Ensembler(nn.Module):
     Compute sum (or average) of outputs of several models.
     """
 
-    def __init__(self, models: List[nn.Module], average=True, outputs=None):
+    def __init__(self, models: List[nn.Module], reduction: str = "mean", outputs: Optional[Iterable[str]] = None):
         """
 
         :param models:
-        :param average:
+        :param reduction: Reduction key ('mean', 'sum', 'gmean', 'hmean' or None)
         :param outputs: Name of model outputs to average and return from Ensembler.
             If None, all outputs from the first model will be used.
         """
         super().__init__()
         self.outputs = outputs
         self.models = nn.ModuleList(models)
-        self.average = average
+        self.reduction = reduction
 
     def forward(self, *input, **kwargs):  # skipcq: PYL-W0221
-        output_0 = self.models[0](*input, **kwargs)
-        num_models = len(self.models)
+        outputs = [model(*input, **kwargs) for model in self.models]
 
         if self.outputs:
             keys = self.outputs
         else:
-            keys = output_0.keys()
+            keys = outputs[0].keys()
 
-        for index in range(1, num_models):
-            output_i = self.models[index](*input, **kwargs)
+        averaged_output = {}
+        for key in keys:
+            predictions = [output[key] for output in outputs]
+            predictions = torch.stack(predictions)
+            predictions = _deaugment_averaging(predictions, self.reduction)
+            averaged_output[key] = predictions
 
-            # Sum outputs
-            for key in keys:
-                output_0[key].add_(output_i[key])
-
-        if self.average:
-            for key in keys:
-                output_0[key].mul_(1.0 / num_models)
-
-        return output_0
+        return averaged_output
 
 
 class PickModelOutput(nn.Module):
