@@ -9,14 +9,13 @@ from typing import Tuple, List, Optional, Union, Callable, Dict
 
 import torch
 from torch import Tensor, nn
-from torch.nn.functional import interpolate
-from ..utils.support import pytorch_toolbelt_deprecated
+
 from . import functional as F
+from ..utils.support import pytorch_toolbelt_deprecated
 
 __all__ = [
     "GeneralizedTTA",
-    "MultiscaleTTAWrapper",
-    "TTAWrapper",
+    "MultiscaleTTA",
     "d2_image_augment",
     "d2_image_deaugment",
     "d2_labels_deaugment",
@@ -26,10 +25,13 @@ __all__ = [
     "d4_image_deaugment",
     "d4_labels_deaugment",
     "fivecrop_image2label",
+    "fivecrop_image_augment",
+    "fivecrop_label_deaugment",
     "fliplr_image2label",
     "fliplr_image2mask",
     "fliplr_image_augment",
     "fliplr_image_deaugment",
+    "fliplr_labels_deaugment",
     "flips_image_augment",
     "flips_image_deaugment",
     "flips_labels_deaugment",
@@ -41,17 +43,70 @@ __all__ = [
 MaybeStrOrCallable = Optional[Union[str, Callable]]
 
 
-def fliplr_image2label(model: nn.Module, image: Tensor) -> Tensor:
-    """Test-time augmentation for image classification that averages predictions
-    for input image and horizontally flipped one.
-
-    :param model:
-    :param image:
-    :return:
+def _deaugment_averaging(x: Tensor, reduction: MaybeStrOrCallable) -> Tensor:
     """
-    output = model(image) + model(F.torch_fliplr(image))
-    one_over_2 = float(1.0 / 2.0)
-    return output * one_over_2
+    Helper method to average predictions of TTA-ed model.
+    This function assumes TTA dimension is 0, e.g [T, B, C, Ci, Cj, ..]
+    Args:
+        x: Input tensor of shape [T, B, ... ]
+        reduction: Reduction mode ("sum", "mean", "gmean", "hmean", function, None)
+
+    Returns:
+        Tensor of shape [B, C, Ci, Cj, ..]
+    """
+    if reduction == "mean":
+        x = x.mean(dim=0)
+    elif reduction == "sum":
+        x = x.sum(dim=0)
+    elif reduction in {"gmean", "geometric_mean"}:
+        x = F.geometric_mean(x, dim=0)
+    elif reduction in {"hmean", "harmonic_mean"}:
+        x = F.harmonic_mean(x, dim=0)
+    elif callable(reduction):
+        x = reduction(x, dim=0)
+    elif reduction in {None, "None", "none"}:
+        pass
+    else:
+        raise KeyError(f"Unsupported reduction mode {reduction}")
+
+    return x
+
+
+def fivecrop_image_augment(image: Tensor, crop_size: Tuple[int, int]) -> Tensor:
+    """Test-time augmentation for image classification that takes five crops out of input tensor (4 on corners and central)
+    and averages predictions from them.
+
+    :param image: Input image tensor
+    :param crop_size: Crop size. Must be smaller than image size
+    :return: Averaged logits
+    """
+    image_height, image_width = int(image.size(2)), int(image.size(3))
+    crop_height, crop_width = crop_size
+
+    if crop_height > image_height:
+        raise ValueError(f"Tensor height ({image_height}) is less than requested crop size ({crop_height})")
+    if crop_width > image_width:
+        raise ValueError(f"Tensor width ({image_width}) is less than requested crop size ({crop_width})")
+
+    bottom_crop_start = image_height - crop_height
+    right_crop_start = image_width - crop_width
+    crop_tl = image[..., :crop_height, :crop_width]
+    crop_tr = image[..., :crop_height, right_crop_start:]
+    crop_bl = image[..., bottom_crop_start:, :crop_width]
+    crop_br = image[..., bottom_crop_start:, right_crop_start:]
+
+    center_crop_y = (image_height - crop_height) // 2
+    center_crop_x = (image_width - crop_width) // 2
+    crop_cc = image[..., center_crop_y : center_crop_y + crop_height, center_crop_x : center_crop_x + crop_width]
+
+    return torch.cat([crop_tl, crop_tr, crop_bl, crop_br, crop_cc], dim=0,)
+
+
+def fivecrop_label_deaugment(logits: Tensor, reduction: MaybeStrOrCallable = "mean") -> Tensor:
+    crop_tl, crop_tr, crop_bl, crop_br, crop_cc = torch.chunk(logits, 5)
+
+    logits: Tensor = torch.stack([crop_tl, crop_tr, crop_bl, crop_br, crop_cc])
+    return _deaugment_averaging(logits, reduction=reduction)
 
 
 def fivecrop_image2label(model: nn.Module, image: Tensor, crop_size: Tuple) -> Tensor:
@@ -63,39 +118,10 @@ def fivecrop_image2label(model: nn.Module, image: Tensor, crop_size: Tuple) -> T
     :param crop_size: Crop size. Must be smaller than image size
     :return: Averaged logits
     """
-    image_height, image_width = int(image.size(2)), int(image.size(3))
-    crop_height, crop_width = crop_size
-
-    assert crop_height <= image_height
-    assert crop_width <= image_width
-
-    bottom_crop_start = image_height - crop_height
-    right_crop_start = image_width - crop_width
-    crop_tl = image[..., :crop_height, :crop_width]
-    crop_tr = image[..., :crop_height, right_crop_start:]
-    crop_bl = image[..., bottom_crop_start:, :crop_width]
-    crop_br = image[..., bottom_crop_start:, right_crop_start:]
-
-    assert crop_tl.size(2) == crop_height
-    assert crop_tr.size(2) == crop_height
-    assert crop_bl.size(2) == crop_height
-    assert crop_br.size(2) == crop_height
-
-    assert crop_tl.size(3) == crop_width
-    assert crop_tr.size(3) == crop_width
-    assert crop_bl.size(3) == crop_width
-    assert crop_br.size(3) == crop_width
-
-    center_crop_y = (image_height - crop_height) // 2
-    center_crop_x = (image_width - crop_width) // 2
-
-    crop_cc = image[..., center_crop_y : center_crop_y + crop_height, center_crop_x : center_crop_x + crop_width]
-    assert crop_cc.size(2) == crop_height
-    assert crop_cc.size(3) == crop_width
-
-    output = model(crop_tl) + model(crop_tr) + model(crop_bl) + model(crop_br) + model(crop_cc)
-    one_over_5 = float(1.0 / 5.0)
-    return output * one_over_5
+    input_aug = fivecrop_image_augment(image, crop_size)
+    preds_aug = model(input_aug)
+    output = fivecrop_label_deaugment(preds_aug)
+    return output
 
 
 def tencrop_image2label(model: nn.Module, image: Tensor, crop_size: Tuple) -> Tensor:
@@ -154,6 +180,17 @@ def tencrop_image2label(model: nn.Module, image: Tensor, crop_size: Tuple) -> Te
     return output * one_over_10
 
 
+def fliplr_image2label(model: nn.Module, image: Tensor) -> Tensor:
+    """Test-time augmentation for image classification that averages predictions
+    for input image and horizontally flipped one.
+
+    :param model:
+    :param image:
+    :return:
+    """
+    return fliplr_labels_deaugment(model(fliplr_image_augment(image)))
+
+
 def fliplr_image2mask(model: nn.Module, image: Tensor) -> Tensor:
     """Test-time augmentation for image segmentation that averages predictions
     for input image and vertically flipped one.
@@ -223,13 +260,7 @@ def fliplr_image_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean"
 
     image: Tensor = torch.stack([b1, F.torch_fliplr(b2)])
 
-    if reduction == "mean":
-        image = image.mean(dim=0)
-    if reduction == "sum":
-        image = image.sum(dim=0)
-    if callable(reduction):
-        image = reduction(image, dim=0)
-    return image
+    return _deaugment_averaging(image, reduction=reduction)
 
 
 def d2_image_augment(image: Tensor) -> Tensor:
@@ -268,38 +299,26 @@ def d2_image_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean") ->
         [b1, F.torch_rot180(b2), F.torch_fliplr(b3), F.torch_flipud(b4),]
     )
 
-    if reduction == "mean":
-        image = image.mean(dim=0)
-    if reduction == "sum":
-        image = image.sum(dim=0)
-    if callable(reduction):
-        image = reduction(image, dim=0)
-    return image
+    return _deaugment_averaging(image, reduction=reduction)
 
 
-def d2_labels_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean") -> Tensor:
+def d2_labels_deaugment(logits: Tensor, reduction: MaybeStrOrCallable = "mean") -> Tensor:
     """
     Deaugment input tensor (output of the model) assuming the is 2D tensor (See d2_augment).
     Args:
-        image: Tensor of [B * 4, C] shape
+        logits: Tensor of [B * 4, C] shape
         reduction: Reduction model for aggregating outputs. Default is taking mean.
 
     Returns:
         Tensor of [B, C] shape if reduction is not None or "none", otherwise returns de-augmented tensor of
         [4, B, C] shape
     """
-    assert image.size(0) % 4 == 0
+    assert logits.size(0) % 4 == 0
 
-    b1, b2, b3, b4 = torch.chunk(image, 4)
-    image: Tensor = torch.stack([b1, b2, b3, b4])
+    b1, b2, b3, b4 = torch.chunk(logits, 4)
+    logits: Tensor = torch.stack([b1, b2, b3, b4])
 
-    if reduction == "mean":
-        image = image.mean(dim=0)
-    if reduction == "sum":
-        image = image.sum(dim=0)
-    if callable(reduction):
-        image = reduction(image, dim=0)
-    return image
+    return _deaugment_averaging(logits, reduction=reduction)
 
 
 def d4_image_augment(image: Tensor) -> Tensor:
@@ -352,18 +371,13 @@ def d4_labels_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean") -
         Tensor of [B, C] shape if reduction is not None or "none", otherwise returns de-augmented tensor of
         [8, B, C] shape
     """
-    assert image.size(0) % 8 == 0
+    if image.size(0) % 8 != 0:
+        raise RuntimeError("Batch size must be divisable by 8")
 
     b1, b2, b3, b4, b5, b6, b7, b8 = torch.chunk(image, 8)
     image: Tensor = torch.stack([b1, b2, b3, b4, b5, b7, b7, b8])
 
-    if reduction == "mean":
-        image = image.mean(dim=0)
-    if reduction == "sum":
-        image = image.sum(dim=0)
-    if callable(reduction):
-        image = reduction(image, dim=0)
-    return image
+    return _deaugment_averaging(image, reduction=reduction)
 
 
 def d4_image_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean") -> Tensor:
@@ -378,7 +392,8 @@ def d4_image_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean") ->
         [4, B, C, H, W] shape
 
     """
-    assert image.size(0) % 8 == 0
+    if image.size(0) % 8 != 0:
+        raise RuntimeError("Batch size must be divisable by 8")
 
     b1, b2, b3, b4, b5, b6, b7, b8 = torch.chunk(image, 8)
 
@@ -394,14 +409,7 @@ def d4_image_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean") ->
             F.torch_rot90_cw_transpose(b8),
         ]
     )
-
-    if reduction == "mean":
-        image = image.mean(dim=0)
-    if reduction == "sum":
-        image = image.sum(dim=0)
-    if callable(reduction):
-        image = reduction(image, dim=0)
-    return image
+    return _deaugment_averaging(image, reduction=reduction)
 
 
 def flips_image_augment(image: Tensor) -> Tensor:
@@ -439,14 +447,25 @@ def flips_image_deaugment(image: Tensor, reduction: MaybeStrOrCallable = "mean",
             F.torch_flipud(image[batch_size * 2 : batch_size * 3]),
         ]
     )
+    return _deaugment_averaging(image, reduction=reduction)
 
-    if reduction == "mean":
-        image = image.mean(dim=0)
-    if reduction == "sum":
-        image = image.sum(dim=0)
-    if callable(reduction):
-        image = reduction(image, dim=0)
-    return image
+
+def fliplr_labels_deaugment(logits: Tensor, reduction: MaybeStrOrCallable = "mean",) -> Tensor:
+    """
+    Deaugment input tensor (output of the model) assuming the input was fliplr-augmented image (See fliplr_image_augment).
+    Args:
+        logits: Tensor of [B * 2, C] shape
+        reduction: If True performs averaging of 3 outputs, otherwise - summation.
+
+    Returns:
+        Tensor of [B, C, H, W] shape.
+    """
+    if logits.size(0) % 2 != 0:
+        raise RuntimeError("Batch size must be divisable by 2")
+
+    orig, flipped_lr = torch.chunk(logits, 2)
+    logits: Tensor = torch.stack([orig, flipped_lr])
+    return _deaugment_averaging(logits, reduction=reduction)
 
 
 def flips_labels_deaugment(logits: Tensor, reduction: MaybeStrOrCallable = "mean",) -> Tensor:
@@ -459,18 +478,12 @@ def flips_labels_deaugment(logits: Tensor, reduction: MaybeStrOrCallable = "mean
     Returns:
         Tensor of [B, C, H, W] shape.
     """
-    assert logits.size(0) % 3 == 0
+    if logits.size(0) % 3 != 0:
+        raise RuntimeError("Batch size must be divisable by 3")
 
-    b1, b2, b3, b4 = torch.chunk(logits, 4)
-    logits: Tensor = torch.stack([b1, b2, b3, b4])
-
-    if reduction == "mean":
-        logits = logits.mean(dim=0)
-    if reduction == "sum":
-        logits = logits.sum(dim=0)
-    if callable(reduction):
-        logits = reduction(logits, dim=0)
-    return logits
+    orig, flipped_lr, flipped_ud = torch.chunk(logits, 3)
+    logits: Tensor = torch.stack([orig, flipped_lr, flipped_ud])
+    return _deaugment_averaging(logits, reduction=reduction)
 
 
 @pytorch_toolbelt_deprecated("This class is deprecated. Please use GeneralizedTTA instead")
@@ -485,7 +498,7 @@ class TTAWrapper(nn.Module):
 
 
 def ms_image_augment(
-    image: Tensor, size_offsets: List[Union[int, Tuple[int, int]]], mode="bilinear", align_corners=True
+    image: Tensor, size_offsets: List[Union[int, Tuple[int, int]]], mode="bilinear", align_corners=False
 ) -> List[Tensor]:
     """
     Multi-scale image augmentation. This function create list of resized tensors from the input one.
@@ -493,11 +506,15 @@ def ms_image_augment(
     batch_size, channels, rows, cols = image.size()
     augmented_inputs = []
     for offset in size_offsets:
-        # TODO: Add support of tuple (row_offset, col_offset)
-        if offset == 0:
+        if isinstance(offset, (tuple, list)):
+            rows_offset, cols_offset = offset
+        else:
+            rows_offset, cols_offset = offset, offset
+
+        if rows_offset == 0 and cols_offset == 0:
             augmented_inputs.append(image)
         else:
-            scale_size = rows + offset, cols + offset
+            scale_size = rows + rows_offset, cols + cols_offset
             scaled_input = torch.nn.functional.interpolate(
                 image, size=scale_size, mode=mode, align_corners=align_corners
             )
@@ -515,10 +532,6 @@ def ms_labels_deaugment(
         logits: List of tensors of shape [B, C]
         size_offsets:
         reduction:
-        mode:
-        align_corners:
-        stride: Stride of the output feature map w.r.t to model input size.
-        Used to correctly scale size_offsets to match with size of output feature maps
 
     Returns:
 
@@ -526,16 +539,8 @@ def ms_labels_deaugment(
     if len(logits) != len(size_offsets):
         raise ValueError("Number of images must be equal to number of size offsets")
 
-    deaugmented_outputs = torch.stack(logits)
-
-    if reduction == "mean":
-        deaugmented_outputs = deaugmented_outputs.mean(dim=0)
-    if reduction == "sum":
-        deaugmented_outputs = deaugmented_outputs.sum(dim=0)
-    if callable(reduction):
-        deaugmented_outputs = reduction(deaugmented_outputs, dim=0)
-
-    return deaugmented_outputs
+    logits = torch.stack(logits)
+    return _deaugment_averaging(logits, reduction=reduction)
 
 
 def ms_image_deaugment(
@@ -555,7 +560,7 @@ def ms_image_deaugment(
         mode:
         align_corners:
         stride: Stride of the output feature map w.r.t to model input size.
-        Used to correctly scale size_offsets to match with size of output feature maps 
+        Used to correctly scale size_offsets to match with size of output feature maps
 
     Returns:
 
@@ -565,74 +570,23 @@ def ms_image_deaugment(
 
     deaugmented_outputs = []
     for feature_map, offset in zip(images, size_offsets):
-        if offset == 0:
+        if isinstance(offset, (tuple, list)):
+            rows_offset, cols_offset = offset
+        else:
+            rows_offset, cols_offset = offset, offset
+
+        if rows_offset == 0 and cols_offset == 0:
             deaugmented_outputs.append(feature_map)
         else:
             batch_size, channels, rows, cols = feature_map.size()
-            # TODO: Add support of tuple (row_offset, col_offset)
-            original_size = rows - offset // stride, cols - offset // stride
+            original_size = rows - rows_offset // stride, cols - cols_offset // stride
             scaled_image = torch.nn.functional.interpolate(
                 feature_map, size=original_size, mode=mode, align_corners=align_corners
             )
             deaugmented_outputs.append(scaled_image)
 
     deaugmented_outputs = torch.stack(deaugmented_outputs)
-    if reduction == "mean":
-        deaugmented_outputs = deaugmented_outputs.mean(dim=0)
-    if reduction == "sum":
-        deaugmented_outputs = deaugmented_outputs.sum(dim=0)
-    if callable(reduction):
-        deaugmented_outputs = reduction(deaugmented_outputs, dim=0)
-
-    return deaugmented_outputs
-
-
-@pytorch_toolbelt_deprecated("This class is deprecated. Please use MultiscaleTTA instead")
-class MultiscaleTTAWrapper(nn.Module):
-    """
-    Multiscale TTA wrapper module
-    """
-
-    def __init__(self, model: nn.Module, scale_levels: List[float] = None, size_offsets: List[int] = None):
-        """
-        Initialize multi-scale TTA wrapper
-
-        :param model: Base model for inference
-        :param scale_levels: List of additional scale levels,
-            e.g: [0.5, 0.75, 1.25]
-        """
-        super().__init__()
-        assert scale_levels or size_offsets, "Either scale_levels or size_offsets must be set"
-        assert not (scale_levels and size_offsets), "Either scale_levels or size_offsets must be set"
-        self.model = model
-        self.scale_levels = scale_levels
-        self.size_offsets = size_offsets
-
-    def forward(self, input: Tensor) -> Tensor:
-        h = input.size(2)
-        w = input.size(3)
-
-        out_size = h, w
-        output = self.model(input)
-
-        if self.scale_levels:
-            for scale in self.scale_levels:
-                dst_size = int(h * scale), int(w * scale)
-                input_scaled = interpolate(input, dst_size, mode="bilinear", align_corners=False)
-                output_scaled = self.model(input_scaled)
-                output_scaled = interpolate(output_scaled, out_size, mode="bilinear", align_corners=False)
-                output += output_scaled
-            output /= 1.0 + len(self.scale_levels)
-        elif self.size_offsets:
-            for offset in self.size_offsets:
-                dst_size = int(h + offset), int(w + offset)
-                input_scaled = interpolate(input, dst_size, mode="bilinear", align_corners=False)
-                output_scaled = self.model(input_scaled)
-                output_scaled = interpolate(output_scaled, out_size, mode="bilinear", align_corners=False)
-                output += output_scaled
-            output /= 1.0 + len(self.size_offsets)
-
-        return output
+    return _deaugment_averaging(deaugmented_outputs, reduction=reduction)
 
 
 class GeneralizedTTA(nn.Module):
@@ -714,11 +668,15 @@ class MultiscaleTTA(nn.Module):
         self.deaugment_fn = deaugment_fn
 
     def forward(self, x):
-        ms_inputs = ms_image_augment(x, self.size_offsets)
+        ms_inputs = ms_image_augment(x, size_offsets=self.size_offsets)
         ms_outputs = [self.model(x) for x in ms_inputs]
 
         outputs = {}
-        keys = self.keys or ms_outputs[0].keys()
+        if self.keys is None:
+            keys = ms_outputs[0].keys()
+        else:
+            keys = self.keys
+
         for key in keys:
             deaugment_fn: Callable = self.deaugment_fn[key]
             values = [x[key] for x in ms_outputs]
