@@ -1,11 +1,13 @@
 from __future__ import absolute_import
 
+from .bitempered_loss import *
 from .dice import *
 from .focal import *
+from .focal_cosine import *
 from .jaccard import *
 from .joint_loss import *
 from .lovasz import *
 from .soft_bce import *
 from .soft_ce import *
+from .soft_f1 import *
 from .wing_loss import *
-from .soft_f1 import *
\ No newline at end of file
