@@ -1,3 +1,4 @@
+from torch import nn
 from torch.nn.modules.loss import _Loss
 
 __all__ = ["JointLoss", "WeightedLoss"]
@@ -18,7 +19,11 @@ class WeightedLoss(_Loss):
 
 
 class JointLoss(_Loss):
-    def __init__(self, first, second, first_weight=1.0, second_weight=1.0):
+    """
+    Wrap two loss functions into one. This class computes a weighted sum of two losses.
+    """
+
+    def __init__(self, first: nn.Module, second: nn.Module, first_weight=1.0, second_weight=1.0):
         super().__init__()
         self.first = WeightedLoss(first, first_weight)
         self.second = WeightedLoss(second, second_weight)
