@@ -9,6 +9,7 @@ from .identity import Identity
 __all__ = [
     "ABN",
     "ACT_ELU",
+    "ACT_GELU",
     "ACT_HARD_SIGMOID",
     "ACT_HARD_SWISH",
     "ACT_LEAKY_RELU",
@@ -18,6 +19,9 @@ __all__ = [
     "ACT_RELU",
     "ACT_RELU6",
     "ACT_SELU",
+    "ACT_SIGMOID",
+    "ACT_SILU",
+    "ACT_SOFTPLUS",
     "ACT_SWISH",
     "ACT_SWISH_NAIVE",
     "AGN",
@@ -41,6 +45,7 @@ __all__ = [
 # Activation names
 ACT_CELU = "celu"
 ACT_ELU = "elu"
+ACT_GELU = "gelu"
 ACT_GLU = "glu"
 ACT_HARD_SIGMOID = "hard_sigmoid"
 ACT_HARD_SWISH = "hard_swish"
@@ -52,10 +57,12 @@ ACT_PRELU = "prelu"
 ACT_RELU = "relu"
 ACT_RELU6 = "relu6"
 ACT_SELU = "selu"
+ACT_SIGMOID = "sigmoid"
+ACT_SILU = "silu"
+ACT_SOFTPLUS = "softplus"
 ACT_SWISH = "swish"
 ACT_SWISH_NAIVE = "swish_naive"
 
-
 # This version reduces memory overhead of Swish during training by
 # recomputing torch.sigmoid(x) in backward instead of saving it.
 @torch.jit.script
@@ -227,20 +234,24 @@ class HardSwish(nn.Module):
 def get_activation_block(activation_name: str):
     ACTIVATIONS = {
         ACT_CELU: nn.CELU,
-        ACT_GLU: nn.GLU,
-        ACT_PRELU: nn.PReLU,
         ACT_ELU: nn.ELU,
+        ACT_GELU: nn.GELU,
+        ACT_GLU: nn.GLU,
         ACT_HARD_SIGMOID: HardSigmoid,
         ACT_HARD_SWISH: HardSwish,
         ACT_LEAKY_RELU: nn.LeakyReLU,
         ACT_MISH: Mish,
         ACT_MISH_NAIVE: MishNaive,
         ACT_NONE: Identity,
+        ACT_PRELU: nn.PReLU,
         ACT_RELU6: nn.ReLU6,
         ACT_RELU: nn.ReLU,
         ACT_SELU: nn.SELU,
+        ACT_SILU: nn.SiLU,
+        ACT_SOFTPLUS: nn.Softplus,
         ACT_SWISH: Swish,
         ACT_SWISH_NAIVE: SwishNaive,
+        ACT_SIGMOID: nn.Sigmoid
     }
 
     return ACTIVATIONS[activation_name.lower()]
