@@ -1,29 +1,22 @@
 from __future__ import absolute_import
 
-import torch.nn as nn
 import math
 
-from ..activations import get_activation_module
+import torch.nn as nn
+
+from ..activations import get_activation_block
 
 
-def conv_bn(inp, oup, stride, activation: nn.Module):
-    return nn.Sequential(
-        nn.Conv2d(inp, oup, 3, stride, 1, bias=False),
-        nn.BatchNorm2d(oup),
-        activation(inplace=True),
-    )
+def conv_bn(inp, oup, stride, activation):
+    return nn.Sequential(nn.Conv2d(inp, oup, 3, stride, 1, bias=False), nn.BatchNorm2d(oup), activation())
 
 
-def conv_1x1_bn(inp, oup, activation: nn.Module):
-    return nn.Sequential(
-        nn.Conv2d(inp, oup, 1, 1, 0, bias=False),
-        nn.BatchNorm2d(oup),
-        activation(inplace=True),
-    )
+def conv_1x1_bn(inp, oup, activation):
+    return nn.Sequential(nn.Conv2d(inp, oup, 1, 1, 0, bias=False), nn.BatchNorm2d(oup), activation())
 
 
 class InvertedResidual(nn.Module):
-    def __init__(self, inp, oup, stride, expand_ratio, activation: nn.Module):
+    def __init__(self, inp, oup, stride, expand_ratio, activation):
         super(InvertedResidual, self).__init__()
         self.stride = stride
         assert stride in [1, 2]
@@ -34,11 +27,9 @@ class InvertedResidual(nn.Module):
         if expand_ratio == 1:
             self.conv = nn.Sequential(
                 # dw
-                nn.Conv2d(
-                    hidden_dim, hidden_dim, 3, stride, 1, groups=hidden_dim, bias=False
-                ),
+                nn.Conv2d(hidden_dim, hidden_dim, 3, stride, 1, groups=hidden_dim, bias=False),
                 nn.BatchNorm2d(hidden_dim),
-                activation(inplace=True),
+                activation(),
                 # pw-linear
                 nn.Conv2d(hidden_dim, oup, 1, 1, 0, bias=False),
                 nn.BatchNorm2d(oup),
@@ -48,13 +39,11 @@ class InvertedResidual(nn.Module):
                 # pw
                 nn.Conv2d(inp, hidden_dim, 1, 1, 0, bias=False),
                 nn.BatchNorm2d(hidden_dim),
-                activation(inplace=True),
+                activation(),
                 # dw
-                nn.Conv2d(
-                    hidden_dim, hidden_dim, 3, stride, 1, groups=hidden_dim, bias=False
-                ),
+                nn.Conv2d(hidden_dim, hidden_dim, 3, stride, 1, groups=hidden_dim, bias=False),
                 nn.BatchNorm2d(hidden_dim),
-                activation(inplace=True),
+                activation(),
                 # pw-linear
                 nn.Conv2d(hidden_dim, oup, 1, 1, 0, bias=False),
                 nn.BatchNorm2d(oup),
@@ -68,12 +57,10 @@ class InvertedResidual(nn.Module):
 
 
 class MobileNetV2(nn.Module):
-    def __init__(
-        self, n_class=1000, input_size=224, width_mult=1.0, activation="relu6"
-    ):
+    def __init__(self, n_class=1000, input_size=224, width_mult=1.0, activation="relu6"):
         super(MobileNetV2, self).__init__()
 
-        act = get_activation_module(activation)
+        activation_block = get_activation_block(activation)
 
         block = InvertedResidual
         input_channel = 32
@@ -92,10 +79,8 @@ class MobileNetV2(nn.Module):
         # building first layer
         assert input_size % 32 == 0
         input_channel = int(input_channel * width_mult)
-        self.last_channel = (
-            int(last_channel * width_mult) if width_mult > 1.0 else last_channel
-        )
-        self.layer0 = conv_bn(3, input_channel, 2, act)
+        self.last_channel = int(last_channel * width_mult) if width_mult > 1.0 else last_channel
+        self.layer0 = conv_bn(3, input_channel, 2, activation_block)
 
         # building inverted residual blocks
         for layer_index, (t, c, n, s) in enumerate(interverted_residual_setting):
@@ -104,37 +89,19 @@ class MobileNetV2(nn.Module):
             blocks = []
             for i in range(n):
                 if i == 0:
-                    blocks.append(
-                        block(
-                            input_channel,
-                            output_channel,
-                            s,
-                            expand_ratio=t,
-                            activation=act,
-                        )
-                    )
+                    blocks.append(block(input_channel, output_channel, s, expand_ratio=t, activation=activation_block))
                 else:
-                    blocks.append(
-                        block(
-                            input_channel,
-                            output_channel,
-                            1,
-                            expand_ratio=t,
-                            activation=act,
-                        )
-                    )
+                    blocks.append(block(input_channel, output_channel, 1, expand_ratio=t, activation=activation_block))
 
                 input_channel = output_channel
 
             self.add_module(f"layer{layer_index + 1}", nn.Sequential(*blocks))
 
         # building last several layers
-        self.final_layer = conv_1x1_bn(input_channel, self.last_channel, activation=act)
+        self.final_layer = conv_1x1_bn(input_channel, self.last_channel, activation=activation_block)
 
         # building classifier
-        self.classifier = nn.Sequential(
-            nn.Dropout(0.2), nn.Linear(self.last_channel, n_class)
-        )
+        self.classifier = nn.Sequential(nn.Dropout(0.2), nn.Linear(self.last_channel, n_class))
 
         self._initialize_weights()
 
@@ -167,8 +134,3 @@ class MobileNetV2(nn.Module):
                 n = m.weight.size(1)
                 m.weight.data.normal_(0, 0.01)
                 m.bias.data.zero_()
-
-
-def test():
-    model = MobileNetV2().eval()
-    print(model)
