@@ -1,5 +1,5 @@
 import torch
-import torch.functional as F
+import torch.nn.functional as F
 from torch import nn
 
 __all__ = ["DropBlock2D", "DropBlock3D", "DropBlockScheduled"]
