@@ -16,7 +16,8 @@ def get_lr_decay_parameters(model: nn.Module, learning_rate: float, lr_multiplie
 
     """
     custom_lr_parameters = dict(
-        (group_name, {"params": [], "lr": learning_rate * lr_factor}) for (group_name, lr_factor) in lr_multipliers.items()
+        (group_name, {"params": [], "lr": learning_rate * lr_factor})
+        for (group_name, lr_factor) in lr_multipliers.items()
     )
     custom_lr_parameters["default"] = {"params": [], "lr": learning_rate}
 
